//debugging
var debug = true;

//helper variables
var lastOpenWindowId = null;
var dial_id_reference = null; // global reference to the last call (the one we can cancel)
var dial_id_reference_autohangup_timer;

//main MEMORY object
var memory = {}; 

//initial memory data (some of them will be overwritten by data form storage)
var initialMemory = {};
initialMemory.ready = true;
initialMemory.power_state = false;
initialMemory.default_country_iso = "nl";
initialMemory.caller_id = false;
initialMemory.api_error_foutmelding = "-120";
initialMemory.availableServers = [];
initialMemory.server = {};
initialMemory.user = {};
initialMemory.lastCallNumber = null;
initialMemory.is_connected = false;

//static memory data (this will overwrite data from local storage)
var staticMemory = {};
staticMemory.autoprovisioning_url = "https://redirect.gntel.nl/api/rpc/";
staticMemory.oAuth_client_id = "cCfFSybMZ4PY5jhXSkza2nqNPd7NsRdHSwSdPLQY";
staticMemory.oAuth_scopes = "ctd.dial%20ctd.verify%20user.me%20ctd.accounts";

//main STORAGE object
var storage = {}

storage.sync = function(){
    chrome.storage.local.set({"memory": memory});
    kLog("storing memory into local storage");
}

storage.init = function(){
    chrome.storage.local.get("memory", function (data) {

        //initial data
        memory = initialMemory;

        //merge data from local storage (data)
        if(data.memory !== undefined){
            memory = Object.assign({}, memory, data.memory);
        } 

        //merge data drom static 
        memory = Object.assign({}, memory, staticMemory);

        //handle single-server plugin
        if(memory.autoprovisioning_url == ""){
            memory.server = staticServer;
        }

        //final presistant save
        storage.sync();

        kLog("initial reading from storage to memory");
        
        //TODO run initial stuff like icon etc...
        handleLoggedState();
	});
};

storage.reset = function(){
    chrome.storage.local.clear();
    kLog("erasing browser storage");
    kLog("********* YOU MUST NOW RELOAD PLUGIN **********");	
}

storage.all = function(){
    chrome.storage.local.get(null, function (data) {
        kLog(data);
    });
}

//initial MEMORY/STORAGE stuff
storage.init();


//main listener
chrome.runtime.onMessage.addListener(
    function(request, sender, sendResponse) {

        //log to console
        if(request.command == "log"){
            kLog("request to log via runtime message: "+request.data);
        }

        //memory SET
        if(request.command == "setMemory"){
            memory[request.key]=request.data;
            storage.sync();
            kLog("setting into memory: "+request.key+" = "+request.data);
        }

        //memory GET
        if(request.command == "getMemory"){
            sendResponse(memory);
        }

        //check URL to find oAuth data
        if(request.command == "checkURL" && memory.server.oAuth_redirect_url != undefined){

            //if we got url to match --TODO--
            if(request.data.substring(0, memory.server.oAuth_redirect_url.length) == memory.server.oAuth_redirect_url) {
                
                if(request.data.match(/[&\#]access_token=([\w\/\-]+)/)){
                    memory.user.oAuth_token = request.data.match(/[&\#]access_token=([\w\/\-]+)/)[1];
                    memory.is_connected = true;
                    storage.sync();
                }

                handleLoggedState();
                if (lastOpenWindowId!=null) {
                    chrome.windows.remove(lastOpenWindowId);
                }
                updateUserMetadata();
                lastOpenWindowId = null;

            }
        }

        if(request.command == "power") {
            memory.power_state = !memory.power_state;
            storage.sync();
            chrome.tabs.query({active: true, currentWindow: true}, function (arrayOfTabs) {
                chrome.tabs.reload(arrayOfTabs[0].id);
            });

        }


        if(request.command == "connect") {
            if(memory.autoprovisioning_url==""){
                //static-server plugin
                openLoginTab();
            } else {

                memory.lastUserEmail = request.data;
                storage.sync();
            	json_rpc_call(  memory.autoprovisioning_url,
                                '{"id":"jsonrpc", "jsonrpc":"2.0", "method":"portal.meta.get", "params":["'+memory.oAuth_client_id+'","'+request.data+'"]}',
                                handleAvailableServers,
                                errorCallback);
            }
        }


        if(request.command == "connect_id") {
            memory.server.base_url = memory.availableServers[request.data].base_url;
            memory.server.oAuth_authorize_url = memory.availableServers[request.data].authorize_url;
            memory.server.oAuth_revoke_url = memory.availableServers[request.data].token_revoke_url;
            memory.server.oAuth_redirect_url = memory.availableServers[request.data].redirect_url;
            memory.server.json_rpc_api_url = memory.availableServers[request.data].jsonrpc_api_url;
            memory.server.site_name = memory.availableServers[request.data].site_name;
            memory.server.site_logo = memory.availableServers[request.data].site_logo_main_url;
            memory.availableServers = [];
            storage.sync();
            openLoginTab();
        }

        if(request.command == "reauthorize") {
            openLoginTab();
        }

        if(request.command == "get_available_accounts"){
            json_rpc_api_call({method:'ctd.get_available_accounts',params:'[]'}, handleAvailableAccounts,errorCallback);
        }


        if(request.command == "setDefaultAccount"){
            memory.user.default_account = request.data;
            storage.sync();
        }

        if(request.command == "disconnect"){
            //lets revoke token
            var xhr = new XMLHttpRequest();
            xhr.onreadystatechange = function(event) {
                if (xhr.readyState == 4) {
                    if(xhr.status == 200) {
                        console.log("revoked");
                    } else {
                        console.log("SERVER ERROR: "+xhr.status);
                    }
                }
            };
            try {
                xhr.open('POST', memory.server.oAuth_revoke_url, true);
                xhr.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
                xhr.send('token='+memory.user.oAuth_token+'&client_id='+memory.oAuth_client_id+'');
            } catch(err) {}

            //empty users data
            memory.user = {};
            //empty available servers data
            memory.availableServers = [];

            storage.sync();
            handleLoggedState();

        }

        if(request.command == "initCall"){

            //lastCallNumber
            memory.lastCallNumber = request.data;
            storage.sync();

            //silent Hangup first
            if (dial_id_reference != null) {
                //autohangup handler
                clearTimeout(dial_id_reference_autohangup_timer);
                //cancel that call
                silentCancelCall();
            }

            kLog("Dialing "+request.data+" with account "+memory.user.default_account+" and CLIR is "+memory.caller_id);

            //api call
            json_rpc_api_call({method:'ctd.dial',params:'["'+request.data+'","'+memory.user.default_account+'",'+memory.caller_id+']'}, function(data){

                //if sender.tabs present - request form TAB, else request form PLUGIN POPUP

                if (data.error != null) {
                    kLog("Call could not be set up");
                    if(sender.tab){
                        chrome.tabs.sendMessage(sender.tab.id, {command: "showAlertBox", data: localeText("checkSettings")},{frameId:0}, function(response) {});
                    } else {
                        chrome.tabs.query({active: true, currentWindow: true}, function (arrayOfTabs) {
                            chrome.tabs.sendMessage(arrayOfTabs[0].id, {command: "showAlertBox", data: localeText("checkSettings")},{frameId:0}, function(response) {});
                        });
                        chrome.runtime.sendMessage({command: "callFailed", data: data.error.message}, function(response) {});
                    }
                    return false;
                }

                if (data.result.result == 1) {
                    if(sender.tab){
                        chrome.tabs.sendMessage(sender.tab.id, {command: "showAlertBox", data: localeText("calling")+" "+request.data},{frameId:0}, function(response) {});
                        chrome.tabs.sendMessage(sender.tab.id, {command: "showHangUpBox", data: ""},{frameId:0}, function(response) {});
                    } else {
                        chrome.tabs.query({active: true, currentWindow: true}, function (arrayOfTabs) {
                            chrome.tabs.sendMessage(arrayOfTabs[0].id, {command: "showAlertBox", data: localeText("calling")+" "+request.data},{frameId:0}, function(response) {});
                            chrome.tabs.sendMessage(arrayOfTabs[0].id, {command: "showHangUpBox", data: ""},{frameId:0}, function(response) {});
                        });
                        chrome.runtime.sendMessage({command: "callSuccess"}, function(response) {});
                    }
                    dial_id_reference = data.result.dial_id;
                    dial_id_reference_autohangup_timer = setTimeout(silentCancelCall, 15000);
                }

            }, function(error){
                if(sender.tab){
                    chrome.tabs.sendMessage(sender.tab.id, {command: "showAlertBox", data: error},{frameId:0}, function(response) {});
                } else {
                    chrome.tabs.query({active: true, currentWindow: true}, function (arrayOfTabs) {
                        chrome.tabs.sendMessage(arrayOfTabs[0].id, {command: "showAlertBox", data: error},{frameId:0}, function(response) {});
                    });
                    chrome.runtime.sendMessage({command: "callFailed", data: error}, function(response) {});
                }
            });

        };




        if(request.command == "cancelCall"){

            kLog("Hanging up " + dial_id_reference);

            //autohangup handler
            clearTimeout(dial_id_reference_autohangup_timer);

            //api call
            json_rpc_api_call({method:'ctd.call_cancel',params:'["'+dial_id_reference+'","'+memory.user.default_account+'"]'}, function(data){

                if (data.error != null) {
                    kLog("Call could not be hanged up");
                    return false;
                }

                if (data.result.result == 1 &&  data.result.dial_id == dial_id_reference) {
                    //clear dial id reference
                    dial_id_reference = null;
                    if(sender.tab){
                        kLog(sender);
                        chrome.tabs.sendMessage(sender.tab.id, {command: "showAlertBox", data: localeText("hangingup")},{frameId:0}, function(response) {});
                    } else {
                        chrome.tabs.query({active: true, currentWindow: true}, function (arrayOfTabs) {
                            kLog(arrayOfTabs);
                            chrome.tabs.sendMessage(arrayOfTabs[0].id, {command: "showAlertBox", data: localeText("hangingup")},{frameId:0}, function(response) {});
                            chrome.tabs.sendMessage(arrayOfTabs[0].id, {command: "hideHangUpBox"},{frameId:0}, function(response) {});
                        });
                        chrome.runtime.sendMessage({command: "cancelSuccess"}, function(response) {});
                    }
                }


            }, function(error){
                chrome.tabs.sendMessage(sender.tab.id, {command: "showAlertBox", data: error},{frameId:0}, function(response) {});
            });

        }






    }
);

//***************
//HELPERS
//***************

function kLog(message){
    if(debug==true){
        console.log(message);
    }
}

function localeText(key){
    return chrome.i18n.getMessage(key);
}

function json_rpc_call(url,sendData,successCallback,errorCallback){
	var xhr = new XMLHttpRequest();
	xhr.onreadystatechange = function(event) {
        if (xhr.readyState == 4) {
            if(xhr.status == 200) {
                successCallback(JSON.parse(xhr.responseText));
            } else {	
                errorCallback(xhr.status);
            }
        }
    };
    xhr.open('POST', url, true);
    xhr.setRequestHeader('Content-Type', 'application/json');
    xhr.send(sendData);
}

function json_rpc_api_call(data,successCallback,errorCallback){
    //empty oAuth?
    if (memory.user.oAuth_token == undefined) {
        errorCallback(localeText("pluginLogin"));
        return;
    }

    //if OK return Object with data}
    var xhr = new XMLHttpRequest();
    xhr.onreadystatechange = function(event) {
        if (xhr.readyState == 4) {
            if(xhr.status == 200) {
                //status: oAuth YES
                successCallback(JSON.parse(xhr.responseText));
            } else {
                //status: oAuth NO
                if (xhr.status=="401") {
                    memory.is_connected = false;
                    storage.sync();
                    errorCallback(localeText("sessionExpiredText"));
                    chrome.runtime.sendMessage({command: "expired"}, function(response) {});
                } else {
                    errorCallback(localeText("connectionServerError")+" ("+xhr.status+")");
                }
            }
        }
    };
    xhr.open('POST', memory.server.json_rpc_api_url, true);
    xhr.setRequestHeader('Content-Type', 'application/json');
    xhr.setRequestHeader('Authorization', 'Bearer ' + memory.user.oAuth_token);
    xhr.send('{"id":"jsonrpc", "jsonrpc":"1.0", "method":"'+data.method+'", "params":'+data.params+'}');
}

//this should be global
function errorCallback(data){
	kLog(data);
}


function advErrorCallback(errorCode){
    
    //remove loading info
    $("#loading").hide();
    
    console.log("ERROR CODE: "+errorCode);
    if (errorCode==999) {
        //no oauthKey - no login
        $("#screen_connect").show();
    } else if (errorCode==401) {
        //oAuth key wrong or expired
        $("#screen_connect").show();
    } else if (errorCode>=500 || errorCode==0) {
        $("#screen_connection").show();
    }

};


function handleAvailableServers(data){

    if(data.result.length == 0){
        //no server
        chrome.runtime.sendMessage({command: "noAvailableServers"}, function(response) {});
    }
    if(data.result.length == 1){
        //set data for only 1 available server and open login window
        memory.availableServers = data.result;
        kLog(data.result);
        memory.server.base_url = data.result[0].base_url;
        memory.server.oAuth_authorize_url = data.result[0].authorize_url;
        memory.server.oAuth_revoke_url = data.result[0].token_revoke_url;
        memory.server.oAuth_redirect_url = data.result[0].redirect_url;
        memory.server.json_rpc_api_url = data.result[0].jsonrpc_api_url;
        memory.server.site_name = data.result[0].site_name;
        memory.server.site_logo = data.result[0].site_logo_main_url;
        storage.sync();
        chrome.runtime.sendMessage({command: "closePopup"}, function(response) {});
        openLoginTab();
    }
    if(data.result.length > 1){
        //contact plugin and show choices
        memory.availableServers = data.result;
        chrome.runtime.sendMessage({command: "multipleAvailableServers", data: memory.availableServers}, function(response) {});
    }

}

function handleAvailableAccounts(data){
                        
    if(data.result !== null){

        //this will always be Array, maybe empty - length = 0
        memory.user.available_accounts = data.result.accounts;

        //if 0
        if(data.result.accounts.length == 0){
            memory.user.default_account = undefined;
        }

        //if only 1, set it as default account
        if(data.result.accounts.length == 1){
            memory.user.default_account = data.result.accounts[0].accountcode;
        }

        //more
        if(data.result.accounts.length > 1){
            if(!memory.user.default_account){
                memory.user.default_account = undefined;
            }
        }
    }

    storage.sync();
    chrome.runtime.sendMessage({command: "refresh", data: memory}, function(response) {});

}

function openLoginTab(){
    var loginPopupWidth = 600;
    var loginPopupHeight = 500;
    var loginPopupLeft = (screen.width/2)-(loginPopupWidth/2);
    var loginPopupTop = (screen.height/2)-(loginPopupHeight/2);
    chrome.windows.create({'url': memory.server.oAuth_authorize_url+"?email="+memory.lastUserEmail+"&response_type=token&approval_prompt=force&scope="+memory.oAuth_scopes+"&client_id="+memory.oAuth_client_id+"#email:"+memory.lastUserEmail, 'type': 'popup','width': loginPopupWidth, 'height': loginPopupHeight, 'left': loginPopupLeft, 'top': loginPopupTop}, function(newWindow) {
        lastOpenWindowId = newWindow.id;
    });
}

function updateUserMetadata(){
    json_rpc_api_call({method:'users.me',params:'[]'}, function(data){
        if(data.error === null) {
            memory.user.user_name = data.result.username;
            storage.sync();
        }
    },function(){});
}

function silentCancelCall(){

    if (dial_id_reference != null) {
        console.log("Hanging up silent " + dial_id_reference);

        //api call
        json_rpc_api_call({method:'ctd.call_cancel',params:'["'+dial_id_reference+'"]'}, function(data){

            //clear dial id reference
            dial_id_reference = null;

            //response log
            console.log(data);

        }, function(error){});
    }
}

function handleLoggedState() {

    if (memory.user.oAuth_token === undefined) {
        chrome.browserAction.setIcon({ path : { "128": "img/icon-offline.png" } });
    } else {
        chrome.browserAction.setIcon({ path : { "128": "img/icon-online.png" } });
    }

}
