//send current URL to kernel to find oAuth data if present
chrome.runtime.sendMessage({command: "checkURL", data: window.location.href}, function(response) {});

(function() {
    // var definitions

    // identify our elements with these class names
    var phoneIconClassName = 'ctd-phoneIcon';

    // parsers
    var parsers = {};

    var patterns = {
      "NL": /\(?(0|(\+|00)[1-9]\d{0,2}\s?)(\s?\(\s?0\s?\)\s?)?\(?\d{1,3}\)?\s?(-|\u2013|\u2014|\/|\w)?\s?\d{2,3}\s?\d{2,3}\s?\d{0,2}\s?\d{0,2}(?!\s?\d)/g,
      "DE": /\(?(0|(\+|00)[1-9]\d{0,2}\s?)(\s?\(\s?0\s?\)\s?)?\(?3?\d{1,4}\)?\s?(-|\u2013|\u2014|\/)?\s?\d{2,3}(\s|-|\u2013|\u2014|\/)?\d{2,3}\s?\d{0,2}\s?\d{0,2}(\s?(-|\u2013|\u2014|\/)\s?\d{1,4})?(?!(\s|-|\/)?\d)/g,
      "SK": /\(?(0|(\+|00)[1-9]\d{0,2}\s?)(\s?\(\s?0\s?\)\s?)?\(?\d{1,3}\)?\s?(-|\u2013|\u2014|\/|\w)?\s?\d{2,3}\s?\d{2,3}\s?\d{0,2}\s?\d{0,2}(?!\s?\d)/g,
      "GB": /\(?(0|(\+|00)[1-9]\d{0,2}\s?)(\s?\(\s?0\s?\)\s?)?\(?\d{1,3}\)?\s?(-|\u2013|\u2014|\/|\w)?\s?\d{2,3}(\s|-|\u2013|\u2014|\/|\w)?\d{2,3}\s?\d{0,2}\s?\d{0,2}(?!\s?\d)/g,
      "BE": /\(?(0|(\+|00)[1-9]\d{0,2}\s?)(\s?\(\s?0\s?\)\s?)?\(?\d{1,3}\)?\s?((-|\u2013|\u2014|\w)?\s?\d{2,3}\s?\d{1,3}\s?\d{0,2}?\s?\d{0,3}|\/\d{2,3}\.\d{2,3}\.\d{2})(?!\s?\d)/g,
      "FR": /\(?(0|(\+|00)[1-9]\d{0,2}\s?)(\s?\(\s?0\s?\)\s?)?\(?\d{1,3}\)?\s?(-|\u2013|\u2014|\/|\w)?\s?\d{2,3}\s?\d{2,3}\s?\d{0,2}\s?\d{0,2}(?!\s?\d)/g
    }

    var IBANs = {
      "NL": /NL\s?\d{2}\s?[A-Z]{4}\s?[0-9]{4}\s?\d{4}\s?\d{2}/,
      "DE": /DE\s?\d{2}\s?([0-9]{4}\s?){4}\s?\d{2}/,
      "SK": /SK\s?\d{2}\s?([0-9]{4}\s?){5}/,
      "GB": /GB\s?\d{2}\s?[A-Z]{4}\s?([0-9]{4}\s?){3}\s?\d{2}/,
      "BE": /BE\s?\d{2}\s?\d{4}\s?\d{4}\s?\d{4}/,
      "FR": /FR\s?\d{2}\s?([0-9]{4}\s?){2}([0-9A-Z]{4}\s?){3}[0-9A-Z]{3}/
    }

    var yearRanges = {
      "NL": /(19|20)\d{2}\s?-\s?(19|20)\d{2}/,
      "DE": /(19|20)\d{2}\s?-\s?(19|20)\d{2}/,
      "SK": /(19|20)\d{2}\s?-\s?(19|20)\d{2}/,
      "GB": /(19|20)\d{2}\s?-\s?(19|20)\d{2}/,
      "BE": /(19|20)\d{2}\s?-\s?(19|20)\d{2}/,
      "FR": /(19|20)\d{2}\s?-\s?(19|20)\d{2}/,
    }


    /**
     * Observer: search and insert icons after mutations.
     */
    var observer;
    var parkedNodes = [];
    var handleMutationsTimeout;

    // using an object to check if tagName is disallowed is faster when using `tagName in {}` than using `Array.indexOf(tagname)`
    var blockedTagNames = (function() {
        // tag list based on:
        // https://developer.mozilla.org/en-US/docs/Web/Guide/HTML/HTML5/HTML5_element_list
        var tags = [
            'TITLE',
            'BASE',
            'LINK',
            'META',
            'STYLE',
            'SCRIPT',
            'TEMPLATE',
            'PRE',
            'FIGURE',
            'DATA',
            'TIME',
            'CODE',
            'VAR',
            'SAMP',
            'KBD',
            'SUB',
            'SUP',
            'RUBY',
            'RT',
            'RP',
            'BDI',
            'BR',
            'WBR',
            'IMG',
            'EMBED',
            'OBJECT',
            'PARAM',
            'VIDEO',
            'AUDIO',
            'SOURCE',
            'TRACK',
            'CANVAS',
            'MAP',
            'AREA',
            'SVG',
            'MATH',
            'INPUT',
            'BUTTON',
            'SELECT',
            'DATALIST',
            'OPTGROUP',
            'OPTION',
            'TEXTAREA',
            'KEYGEN',
            'PROGRESS',
            'METER',
            'DETAILS',
            'SUMMARY',
            'MENUITEM',
            'MENU',
        ];

        var disallowed = {};
        for(var i = 0; i < tags.length; i++) {
            disallowed[tags[i]] = null;
        }

        return disallowed;
    })();

    function hasIcon(node) {
        if (node&&(typeof(node)!='undefined')) {
            if (node.className==phoneIconClassName) {
                return true;
            }
        }
        return false;
    }

    /**
     * Skip elements which *probably* wouldn't (or shouldn't) contain a phone number.
     */
    function isBlockedElement(element) {
        if(element.isContentEditable){
            return true;
        }
        if (element.tagName.toUpperCase() in blockedTagNames) {
            return true;
        }
        return false;
    }

    /**
     * Test if `node` should even be processed.
     */
    function skipNode(node) {
        // nodetype 1 == HTML element
        if(node.nodeType == 1 && isBlockedElement(node)) {
            return true;
        }

        // nodetype 3 == Text node
        if(node.nodeType == 3) {
            // skip empty nodes
            if(node.data.trim().length === 0) {
                return true;
            }
        }

        var parentElement = node.parentElement;
        if(parentElement) {
            // skip invisible elements,
            // Sizzle: an element is invisible when it has no height or width
            if(!(parentElement.offsetWidth > 0 || parentElement.offsetHeight > 0)) {
                return true;
            }

            if(isBlockedElement(parentElement)) {
                return true;
            }

            if(hasIcon(node.nextElementSibling)) {
                return true;
            }
        }

        return false;
    };

    /**
     * Walk the DOM and apply fn for every node.
     */
    function walkTheDOM(node, fn) {
        if(node) do {
            if(skipNode(node)) {
                continue;
            }

            // in case fn replaces node, skip it
            var returnNode = fn(node);
            if(returnNode != node) {
                node = returnNode;
                continue;
            }

            walkTheDOM(node.firstChild, fn);
        } while((node = node.nextSibling));
    };

    /**
     * Universal parser
     * Scan data for phone numbers matching countyISO format
     * and return the node in the DOM.
     */

    function uniParser(countryISO){

        var pattern = patterns[countryISO];
        var IBAN = IBANs[countryISO];
        var yearRange = yearRanges[countryISO];
        var matches;

        /**
         * Find phone number matches in `text`.
         */
        var parse = function(text) {
            matches = [];
            // very crude early fails:

            // 1) we need at least 8 digits
            var digits = text.match(/\d/g);
            if(digits === null || digits.length < 8) {
                return matches;
            } else {
                // 2) we need at least two consecutive integers!
                if(!new RegExp(/\d{2,}/g).test(text)) {
                    return matches;
                }
            }

            while((match = pattern.exec(text)) != null) {
                // check if we need to block based on preceding text
                var block = false;
                if (match.index>0) {
                    var preceding = text.slice(match.index-1, match.index);
                    if (preceding>='0'&&preceding<='9') block =true;
                }
                // check for IBAN
                if (IBAN.test(text)||yearRange.test(text)) {
                    block = true;
                }

                if (!block) {
                    // remove optional zero and replace nondigits
                    response = gntel.phoneNumberValidator(match[0], countryISO);
                    if (response.isNumberValid) {
                        matches.push({
                            start: match.index,
                            end: match.index + match[0].length,
                            number: response.E164,
                        });
                    }
                }
            }
            return matches;
        }

        var isBlockingNode = function(node) {
            var blocking = false;
            if(node) {
                var text = node.textContent.trim().toLowerCase();
                if(text == 'fax' ||
                   text == 'fax.' ||
                   text == 'fax:' ||
                   text == 'f' ||
                   text == 'f.' ||
                   text == 'f:') {
                    blocking = true;
                }
            }

            return blocking;
        };

        // exposed functions
        return {
            parse: parse,
            isBlockingNode: isBlockingNode,
        };

    }


    /**
     * Create an HTML element containing an anchor with a phone icon with
     * the phone number in a data attribute.
     */
    function createIconElement(number) {
        return $('<a>', {
            'href': 'tel:' + number,
            'class': phoneIconClassName,
        });
    }

    function doInsert(root) {

        //check power state
        chrome.runtime.sendMessage({command: "getMemory"}, function(response) {

            //reading whole memory
            var memory = response;

            if (memory.power_state != true ) {
                return;
            }

            var pause = !!root;

            if(pause) {
                stop_observer();
            }

            root = root || document.body;
            var tld = document.location.hostname.split('.').splice(-1,1);
            tld = tld[0];
            var country = "NL";  // default
            if (tld=="nl") {
                country = "NL";
            } else if (tld=="de") {
                country = "DE";
            } else if (tld=="sk") {
                country = "SK";
            } else if (tld=="gb") {
                country = "GB";
            } else if (tld=="uk") {
                country = "GB";
            } else if (tld=="be") {
                country = "BE";
            } else if (tld=="fr") {
                country = "FR";
            } else {
                country = memory.default_country_iso.toUpperCase();
            }

            // walk the DOM looking for elements to parse
            walkTheDOM(root, function(node) {
                // is it a Text node?
                if(node.nodeType === 3) {
                    // does it have non whitespace text content?
                    var text = node.data.trim();
                    if(text.length > 0) {
                        var matches = [];
                        // scan using the (proper parser) uniParser
                        var parser = uniParser(country);
                        matches = parser.parse(node.data);

                        // insert icons after every phone number
                        if(matches.length) {
                            if(!parser.isBlockingNode(node.previousElementSibling) &&
                                    !parser.isBlockingNode(node.parentNode.previousElementSibling)&&
                                    !hasIcon(node.nextElementSibling)) {

                                // loop backwards to make indexes work
                                matches.reverse().forEach(function(match) {
                                    var iconElement = $(createIconElement(match.number));

                                    if (match.end < node.length) {
                                        var after = node.splitText(match.end);
                                        iconElement.insertBefore(after);
                                    } else {
                                        iconElement.insertAfter(node);
                                    }
                                });
                            }
                        }
                    }
                }
                return node;
            });

            if(pause) {
                start_observer();
            }

            $('.' + phoneIconClassName).click(function(event) {

                // remove focus
                $(this).blur();

                // don't do anything with this click in the actual page
                event.preventDefault();
                event.stopPropagation();
                event.stopImmediatePropagation();

                chrome.runtime.sendMessage({command: "initCall", data: $(this).attr('href').substring(4)}, function(response) {});

            });

        });
    }


    function undoInsert() {
        // remove icons from page
        $('.'+phoneIconClassName).remove();
    }

    /**
     * Process parked DOM mutations.
     */
    function handleMutations() {
        // copy and clear parkedNodes
        var _parkedNodes = parkedNodes.slice();
        parkedNodes = [];

        // handle mutations if it probably isn't too much to handle
        // (current limit is totally random)
        if(_parkedNodes.length < 1500) {
            _parkedNodes.forEach(function(node) {
                doInsert(node);
            });
        }
    }

    /**
     * Observer start: listen for DOM mutations and let `handleMutations`
     * process them.
     */
    function start_observer() {
        if(!observer) {
            observer = new MutationObserver(function(mutations) {
                if(handleMutationsTimeout) {
                    // don't handle the mutations yet after all
                    clearTimeout(handleMutationsTimeout);
                }

                mutations.forEach(function(mutation) {
                    // filter mutations to park
                    if(mutation.addedNodes.length) {
                        $.each(mutation.addedNodes, function(index, addedNode) {
                            if(!skipNode(addedNode)) {
                                parkedNodes.push(addedNode);
                            }
                        });
                    } else if(!mutation.removedNodes.length && mutation.target) {
                        if(!skipNode(mutation.target)) {
                            parkedNodes.push(mutation.target);
                        }
                    }
                });

                // assuming nothing happens, scan the nodes in 500 ms - after
                // this the page should've been done dealing with the mutations
                if(parkedNodes.length) {
                    handleMutationsTimeout = setTimeout(function() {
                        handleMutations();
                    }, 500);
                }
            });
        }

        if(observer) {
            observer.observe(document.body, {
                attributes:true,
                // characterData: true,  // disabled for now
                childList: true,
                subtree: true,
            });
        }
    }

    /**
     * Observer stop: simply stop listening to DOM mutations.
     */
    function stop_observer() {
        if(observer) {
            observer.disconnect();
        }
    }

    chrome.runtime.onMessage.addListener(
        function(request, sender, sendResponse) {
            if(request == 'page.observer.stop') {
                console.info('page.observer.stop');

                // stop listening to DOM mutations
                stop_observer();

                // remove icons
                undoInsert();
            }
        }
    );

    // signal this script has been loaded and ready to look for phone numbers
    chrome.runtime.sendMessage('page.observer.ready', function(response) {
        var doRun = function() {
            // insert icons
            doInsert();

            // start listening to DOM mutations
            start_observer();
        };

        if(window != window.top && !(document.body.offsetWidth > 0 || document.body.offsetHeight > 0)) {
            // this hidden iframe might become visible, wait for this to happen
            $(window).on('resize', function() {
                doRun();

                // no reason to wait for more resize events
                $(window).off('resize');
            });
        } else {
            doRun();
        }
    });

    var alertBoxTimeout;
    var hangUpBoxTimeout;

    $("body").append('<div id="ctd-alertBox"><div id="ctd-alertBoxContent"></div></div>');
    $("body").append('<div id="ctd-hangUpBox"></div>');
    $("#ctd-hangUpBox").click(function(){
        clearTimeout(hangUpBoxTimeout);
        chrome.runtime.sendMessage({command: "cancelCall"}, function(response) {});
        $('#ctd-hangUpBox').css('top', -100);
    });

    chrome.runtime.onMessage.addListener(
    function(request, sender, sendResponse) {

        if(request.command == "showAlertBox"){

            clearTimeout(hangUpBoxTimeout);
            clearTimeout(alertBoxTimeout);
            $("#ctd-alertBoxContent").text(request.data);
            $("#ctd-alertBox").css('top', 20);
            $("#ctd-alertBox").css('opacity', 1);
            alertBoxTimeout = setTimeout(function() {
                $('#ctd-alertBox').css('top', -100);
                $("#ctd-alertBox").css('opacity', 0);
            }, 4000);
        };

        if(request.command == "hideHangUpBox"){
            $('#ctd-hangUpBox').css('top', -100);
        }

        if(request.command == "showHangUpBox"){
            $("#ctd-hangUpBox").css('top', 100);
            $("#ctd-hangUpBox").css('opacity', 1);
            hangUpBoxTimeout = setTimeout(function() {
                $('#ctd-hangUpBox').css('top', -100);
                $("#ctd-hangUpBox").css('opacity', 0);
            }, 5000);
        }

    });

    //TODO
    chrome.runtime.sendMessage({command: "handleLoggedState"}, function(response) {});


})();
