//main MEMORY object (will be filled from kernel memory)
var memory = {};

var popupHangupTimeout;

function localeText(key){
    return chrome.i18n.getMessage(key);
}

function handleTexts(){
    $(".connectText").text(localeText("connectText"));
    $(".connectButton").text(localeText("connectButton"));
    $(".reauthorizeButton").text(localeText("reauthorizeButton"));
    $(".switchButton").text(localeText("switchButton"));
    $(".sessionExpiredText").text(localeText("sessionExpiredText"));
    $(".disconnectButton").text(localeText("disconnectButton"));
    $("#power_on_button").text(localeText("powerOnButton"));
    $(".defaultAccountText").text(localeText("defaultAccountText"));
    $(".singleAccountText").text(localeText("singleAccountText"));
    $(".callerIdText").text(localeText("callerIdText"));
    $(".defaultCountryText").text(localeText("defaultCountryText"));
    $(".defaultCountryHint").text(localeText("defaultCountryHint"));
    $(".ctdText").text(localeText("ctdText"));
    $(".connectionServerText").text(localeText("connectionServerError"));
    $("#freeSeatingOptGroup").attr("label",localeText("freeSeatingOptGroupText"));
    $("#phoneAccountsOptGroup").attr("label",localeText("phoneAccountsOptGroupText"));
    $("#loading").text(localeText("loadingText"));
    $(".multipleServersText").text(localeText("multipleServersText"));
    $(".noServersText").text(localeText("noServersText"));

    if (memory.user && memory.user.user_name !== undefined && memory.server && memory.server.site_name !== undefined) {
        $("#userEmail").html(memory.user.user_name);
        if (memory.autoprovisioning_url || memory.server.site_logo ){
            $("#logo").css("background-image", "url("+memory.server.site_logo+")");
        }
    }

    if(memory.lastUserEmail != undefined){
        $("#loginEmail").val(memory.lastUserEmail);
    }

    if (!memory.autoprovisioning_url){
        $("#loginEmail").hide();
    }

    $("#phonenumber").val(memory.lastCallNumber);
}

//handle FLAG in settings
function handleDefaultCountryFlag(){
    $("#default_country img").attr("src","img/flag_"+memory.default_country_iso+".png");
}

//CLIR
function handleCallerIDSwitch(){
    $("#callerIdSwitch").prop('checked', memory.caller_id);
}

function setDefaultCountry(countryISO){
    memory.default_country_iso = countryISO;
    chrome.runtime.sendMessage({command: "setMemory", key: "default_country_iso", data: countryISO}, function(response) {});
    handleDefaultCountryFlag();
    handleScreen("#screen_settings");
    //reload current TAB
    chrome.tabs.query({active: true, currentWindow: true}, function (arrayOfTabs) {
        chrome.tabs.reload(arrayOfTabs[0].id);
    });
}

function json_rpc_api_call(data, successCallback, errorCallback){

    //empty oAuth?
    if (memory.user.oAuth_token == undefined) {
        errorCallback(999);
        return;
    }

    //if OK return Object with data}
    var xhr = new XMLHttpRequest();
    xhr.onreadystatechange = function(event) {
        if (xhr.readyState == 4) {
            if(xhr.status == 200) {
                //status: oAuth YES
                successCallback(JSON.parse(xhr.responseText));
            } else {
                //status: oAuth NO
                errorCallback(xhr.status);
            }
        }
    };
    xhr.open('POST', memory.server.json_rpc_api_url, true);
    xhr.setRequestHeader('Content-Type', 'application/json');
    xhr.setRequestHeader('Authorization', 'Bearer ' + memory.user.oAuth_token);
    xhr.send('{"id":"jsonrpc", "jsonrpc":"1.0", "method":"'+data.method+'", "params":'+data.params+'}');

}

function handleScreen(customScreen){

    //hide all first-child DIVs, all "screen_"
    $("#screens > div").hide();

    //hide all action elements
    $("#settings_button").hide();
    $("#power_off_button").hide();
    $("#back_button").hide();
    $("#loading").hide();

    //handle power button
    if (memory.power_state != true) {
        //show OFF screen
        $("#screen_suspend").show();
        return;
    } else {
        //power button
        $("#power_off_button").show();
    }

    //custom screen
    if (customScreen != undefined) {
        $(customScreen).show();
        $("#back_button").show();
        return;
    }

    //unexisting user
    if(memory.user.oAuth_token == undefined){
        $("#screen_connect").show();
        return;
    }

    //existing user, BUT expired session
    if(memory.is_connected == false){
        $("#screen_expired").show();
        return;
    }


    handleDefaultCountryFlag();
    handleCallerIDSwitch();


    if(memory.user.available_accounts == undefined){
        $("#loading").show();
        chrome.runtime.sendMessage({command: "get_available_accounts"}, function(response) {});
        return;
    }

    if(memory.user.available_accounts == []){
        handleScreen("#screen_no_available_accounts");
        $("#settings_button").show();
        return;
    }

    //prepare data for setting page
    if(memory.user.available_accounts.length == 1){
        $("#single_account").show();
        $("#single_account .singleAccountId").remove();
        $("#single_account").append('<p class="singleAccountId"><b>'+memory.user.default_account+'</b></p>');
    } else {
        $("#noAccountOption").remove();
        $("#accountSelector").prepend('<option id="noAccountOption" value="">-</option>');

        $("#freeSeatingOptGroup").empty();
        $("#phoneAccountsOptGroup").empty();

        //loop for all accounts
        for (var key in memory.user.available_accounts) {

            //set current account in loop
            currentAccount = memory.user.available_accounts[key];

            if (memory.user.default_account == currentAccount.accountcode){
                currentAccount.status = "selected";
                $("#noAccountOption").remove();//is not needed anymore
            } else {
                currentAccount.status = "";
            }

            //add into select lost, mark as selected if exists in storage
            if (currentAccount.type == "f") {
                 $("#freeSeatingOptGroup").append('<option '+currentAccount.status+' value="'+currentAccount.accountcode+'">'+currentAccount.accountcode+'</option>');
            }
            if (currentAccount.type == "p") {
                 $("#phoneAccountsOptGroup").append('<option '+currentAccount.status+' value="'+currentAccount.accountcode+'">'+currentAccount.accountcode+'</option>');
            }

        }

        //remove unneeded options groups
        $("#freeSeatingOptGroup:empty").remove();
        $("#phoneAccountsOptGroup:empty").remove();
        $("#default_account").show();
    }

    if(memory.user.default_account == undefined){
        $("#screen_settings").show();
    } else {
        $("#screen_ctd").show();
        $("#settings_button").show();
    }

}


function connect(){
    if($("#loginEmail").val() || memory.autoprovisioning_url==""){
        $("#screens > div").hide();
        $("#loading").show();
        chrome.runtime.sendMessage({command: "connect", data: $("#loginEmail").val()}, function(response) {});
        //kernel.js decides about closing popup.html
    }
}

function connect_id(server_id){
    chrome.runtime.sendMessage({command: "connect_id", data: server_id}, function(response) {});
    window.close();//closing popup.html
}

function reauthorize(){
    chrome.runtime.sendMessage({command: "reauthorize", data: ""}, function(response) {});
    window.close();//closing popup.html
}

function disconnect(){
    chrome.runtime.sendMessage({command: "disconnect"}, function(response) {});
    window.close();//closing popup.html
}

function power(){
    chrome.runtime.sendMessage({command: "power"}, function(response) {});
    window.close();//closing popup.html
}

function initCall(){
    chrome.runtime.sendMessage({command: "initCall", data: $("#phonenumber").val()}, function(response) {});
    memory.lastCallNumber = $("#phonenumber").val();
}

function cancelCall(){
    chrome.runtime.sendMessage({command: "cancelCall", data: $("#phonenumber").val()}, function(response) {});
}

//helper listener
chrome.runtime.onMessage.addListener(
    function(request, sender, sendResponse) {
        
        if(request.command == "closePopup") {
            window.close();
        }

        if(request.command == "noAvailableServers"){
            handleScreen("#screen_no_servers");
        }

        if(request.command == "multipleAvailableServers"){
            $("#multipleServersChoices").html("");
            $.each( request.data, function( key, value ) {
                $("#multipleServersChoices").append('<button data-id="'+key+'">'+value.site_name+'</button>');
            });
            $("#multipleServersChoices button").click(function(){
                connect_id($(this).data("id"));
            })
            handleScreen("#screen_multiple_servers");
        }

        if(request.command == "expired"){
            handleScreen("#screen_expired");
            $("#back_button").hide();
        }

        if(request.command == "refresh"){
            memory = request.data;
            handleScreen();
        }

        if(request.command == "callSuccess"){
            $("#status").hide();
            $("#phonenumber").slideUp();
            $(".ctdText").text(localeText("dialingText")+" "+$("#phonenumber").val());
            $("#cancel").prop("disabled", false);
            $("#cancel").removeClass("disabled");

            popupHangupTimeout = setTimeout(function(){
                $(".ctdText").text(localeText("ctdText"));
                $("#phonenumber").slideDown();
                $("#cancel").prop("disabled", true);
                $("#cancel").addClass("disabled");
            }, 5000);
        }

        if(request.command == "callFailed"){
            $("#status").html("ERROR: "+request.data);
            $("#status").show();
        }

        if(request.command == "cancelSuccess"){
            clearTimeout(popupHangupTimeout);

            $(".ctdText").text(localeText("ctdText"));
            $("#phonenumber").slideDown();
            $("#cancel").prop("disabled", true);
            $("#cancel").addClass("disabled");
        }

    }
);


//main run
document.addEventListener('DOMContentLoaded', function() {

    chrome.runtime.sendMessage({command: "getMemory"}, function(response) {

        //reading whole memory
        memory = response;

        //always read accounts from server
        delete memory.user.available_accounts;

        // attach event handlers
        $("#dial").click(initCall);
        $("#cancel").click(cancelCall);
        $("#disconnect_button").click(disconnect);
        $("#switch_button").click(disconnect);
        $("#power_off_button").click(power);
        $("#power_on_button").click(power);
        $("#connect_button").click(connect);
        $("#reauthorize_button").click(reauthorize);

        $("#settings_button").click(function() {
            handleScreen("#screen_settings");
        });

        $("#back_button").click(function() {
            handleScreen();
        });

        $("#default_country").click(function() {
            handleScreen("#screen_country");
        });

        //flags
        $(".flag_nl").click(function() {
            setDefaultCountry('nl');
        });

        $(".flag_de").click(function() {
            setDefaultCountry('de');
        });

        $(".flag_sk").click(function() {
            setDefaultCountry('sk');
        });

        $(".flag_gb").click(function() {
            setDefaultCountry('gb');
        });

        $(".flag_be").click(function() {
            setDefaultCountry('be');
        });

        $(".flag_fr").click(function() {
            setDefaultCountry('fr');
        });

        document.getElementById("phonenumber").addEventListener("keydown", function (e) {
            if (e.keyCode === 13) {
                initCall();
            }
        });

        document.getElementById("loginEmail").addEventListener("keydown", function (e) {
            if (e.keyCode === 13) {
                connect();
            }
        });

        $("#accountSelector").change(function(){
            chrome.runtime.sendMessage({command: "setDefaultAccount", data: $("#accountSelector").val()}, function(response) {});
            memory.user.default_account = $("#accountSelector").val();
            $("#noAccountOption").remove();
            $("#back_button").show();
        });

        $("#callerIdSwitch").change(function(){
            chrome.runtime.sendMessage({command: "setMemory", key: "caller_id", data: $("#callerIdSwitch").is(':checked')}, function(response) {});
            memory.caller_id = $("#callerIdSwitch").is(':checked');
        });

        handleTexts();
        handleScreen();

    });
});
